<?php
/**
 * Copyright © 2016 Affilae. All rights reserved.
 */
namespace Affilae\Tracking\Helper;

use Affilae\Tracking\Model\RuleFactory;

class Data extends \Magento\Framework\App\Helper\AbstractHelper
{
    /**
     * System config XML paths
     */
    const XML_PATH_ENABLED    = 'affilae/tracking/enabled';
    const XML_PATH_PID = 'affilae/tracking/pid';

    /**
     * Affilae Tracking registry names
     */
    const CONVERSION_ID_REGISTRY_NAME         = 'affilae_tracking_conversion_id';
    const CONVERSION_VALUE_REGISTRY_NAME      = 'affilae_tracking_conversion_value';
    const CONVERSION_PAYMENT_REGISTRY_NAME    = 'affilae_tracking_conversion_payment';
    const CONVERSION_ITEMS_REGISTRY_NAME      = 'affilae_tracking_conversion_items';
    const CONVERSION_CATEGORIES_REGISTRY_NAME = 'affilae_tracking_conversion_categories';
    const CONVERSION_VOUCHERS                 = 'affilae_tracking_conversion_vouchers';

    const CONVERSION_ID_DEFAULT      = 0;
    const CONVERSION_VALUE_DEFAULT   = 0;
    const CONVERSION_PAYMENT_DEFAULT = 'online';

    /**
     * @var \Magento\Framework\Registry
     */
    protected $_registry;

    /**
     * @var \Affilae\Tracking\Model\RuleFactory
     */
    protected $_ruleFactory;

    /**
     * @param \Magento\Framework\App\Helper\Context $context
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Framework\Registry $registry,
        \Affilae\Tracking\Model\RuleFactory $ruleFactory
    ) {
        parent::__construct($context);
        $this->_registry = $registry;
        $this->_ruleFactory = $ruleFactory;
    }

    /**
     * Check if Affilae is enabled
     *
     * @param null|string|bool|int|Store $store
     *
     * @return bool
     */
    public function isTrackingEnabled($store = null)
    {
        return $this->scopeConfig->isSetFlag(
            self::XML_PATH_ENABLED,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $store
        );
    }

    /**
     * Retrieve Affilae program id
     *
     * @param null|string|bool|int|Store $store
     *
     * @return string
     */
    public function getProgramId($store = null)
    {
        return $this->scopeConfig->getValue(
            self::XML_PATH_PID,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $store
        );
    }

    /**
     * Get the order id from the registry
     *
     * @return string
     */
    public function getOrderId()
    {
        $id = (string) $this->_registry->registry(self::CONVERSION_ID_REGISTRY_NAME);

        return empty($id) ? self::CONVERSION_ID_DEFAULT : $id;
    }

    /**
     * Get the order total from the registry
     *
     * @return double
     */
    public function getOrderTotal()
    {
        $total = (double) $this->_registry->registry(self::CONVERSION_VALUE_REGISTRY_NAME);

        return empty($total) ? self::CONVERSION_VALUE_DEFAULT : $total;
    }

    /**
     * Get the order payment from the registry
     *
     * @return string
     */
    public function getOrderPayment()
    {
        $payment = $this->_registry->registry(self::CONVERSION_PAYMENT_REGISTRY_NAME);

        switch ($payment) {
            case 'Check / Money order':
                $payment = 'cheque';
                break;

            default:
                $payment = self::CONVERSION_PAYMENT_DEFAULT;
                break;
        }

        return $payment;
    }

    /**
     * Get the order categories from the registry
     *
     * @return string
     */
    public function getOrderCategories()
    {
        $categories = $this->_registry->registry(self::CONVERSION_CATEGORIES_REGISTRY_NAME);

        return $categories;
    }

    /**
     * Get the order items from the registry
     *
     * @return string
     */
    public function getOrderItems()
    {
        $items = $this->_registry->registry(self::CONVERSION_ITEMS_REGISTRY_NAME);

        return $items;
    }

    /**
     * Get order vouchers
     *
     * @return string
     */
    public function getOrderVouchers()
    {
        $vouchers = $this->_registry->registry(self::CONVERSION_VOUCHERS);

        return $vouchers;
    }

    /**
     * Get the tracking rules
     *
     * @return array
     */
    public function getTrackingRules()
    {
        $orderTotal = $this->getOrderTotal();
        $orderTotalRemaining = $orderTotal;

        $rules = array();
        $categoryRules = array();
        $genericRules = array();

        $collection = $this->_ruleFactory->create()->getCollection();

        $collection
            ->addFieldToFilter(
                'status',
                ['eq' => 1]
            )
            ->getSelect()
            ->order('rule_id DESC');

        foreach ($collection as $item) {
            $ruleId         = $item->getRuleId();
            $ruleCategories = $item->getParentCategories();

            if ($ruleCategories->count() > 0) {
                // Only add the first category
                foreach ($ruleCategories as $ruleCategory) {
                    $categoryRules[$ruleCategory->getEntityId()] = $item;
                    break;
                }
            } else {
                $genericRules[] = $item;
            }
        }

        if (count($categoryRules) > 0) {
            $orderCategories = $this->getOrderCategories();
            $categoryTotal   = [];

            $items = $this->getOrderItems();

            foreach ($items as $item) {
                $product = $item->getProduct();
                $productId = $product->getId();
                $productCategories = $product->getCategoryCollection();

                foreach ($productCategories as $productCategory) {
                    $categoryId = $productCategory->getEntityId();
                    break;
                }

                if (array_key_exists($categoryId, $categoryRules)) {
                    $rule = $categoryRules[$categoryId];
                    $lineTotal = $item->getRowTotal();

                    $orderTotalRemaining -= (float) $lineTotal;

                    $rules[] = array(
                        'code' => $rule->getCode(),
                        'amount' => $lineTotal
                    );
                }
            }

            if (count($genericRules) > 0 && $orderTotalRemaining > 0) {
                $rule = $genericRules[0];

                $rules[] = array(
                    'code' => $rule->getCode(),
                    'amount' => $orderTotal
                );
            }
        } elseif (count($genericRules) > 0) {
            foreach ($genericRules as $genericRule) {
                $rules[] = array(
                    'code' => $genericRule->getCode(),
                    'amount' => $orderTotal
                );
            }
        }

        return $rules;
    }
}
