<?php
/**
 * Copyright © 2016 Affilae. All rights reserved.
 */
namespace Affilae\Tracking\Model;


/**
 * Rule model
 */
class Rule extends \Magento\Framework\Model\AbstractModel
{
    /**
     * Rules's Statuses
     */
    const STATUS_ENABLED = 1;
    const STATUS_DISABLED = 0;

    /**
     * Prefix of model events names
     *
     * @var string
     */
    protected $_eventPrefix = 'affilae_tracking_rule';

    /**
     * Parameter name in event
     *
     * In observe method you can use $observer->getEvent()->getObject() in this case
     *
     * @var string
     */
    protected $_eventObject = 'tracking_rule';

    /**
     * @var \Magento\Catalog\Model\ResourceModel\Category\CollectionFactory
     */
    protected $_categoryCollectionFactory;

    /**
     * @var \Magento\Catalog\Model\ResourceModel\Category\Collection
     */
    protected $_parentCategories;

    /**
     * Initialize dependencies.
     *
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Catalog\Model\ResourceModel\Category\CollectionFactory $categoryCollectionFactory
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource $resource
     * @param \Magento\Framework\Data\Collection\AbstractDb $resourceCollection
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Catalog\Model\ResourceModel\Category\CollectionFactory $categoryCollectionFactory,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        parent::__construct($context, $registry, $resource, $resourceCollection, $data);

        $this->_categoryCollectionFactory = $categoryCollectionFactory;
    }

    /**
     * Initialize resource model
     *
     * @return void
     */
    protected function _construct()
    {
        $this->_init('Affilae\Tracking\Model\ResourceModel\Rule');
    }

    /**
     * Retrieve model title
     * @param  boolean $plural
     * @return string
     */
    public function getOwnTitle($plural = false)
    {
        return $plural ? 'Rule' : 'Rules';
    }

    /**
     * Retrieve true if rule is active
     * @return boolean [description]
     */
    public function isActive()
    {
        return ($this->getStatus() == self::STATUS_ENABLED);
    }

    /**
     * Retrieve available rule statuses
     * @return array
     */
    public function getAvailableStatuses()
    {
        return [
            self::STATUS_DISABLED => __('Disabled'),
            self::STATUS_ENABLED => __('Enabled')
        ];
    }

    /**
     * Check if rule identifier exist for specific store
     * return rule id if rule exists
     *
     * @param string    $identifier
     * @param int       $storeId
     *
     * @return int
     */
    public function checkIdentifier($identifier, $storeId)
    {
        return $this->_getResource()->checkIdentifier($identifier, $storeId);
    }

    /**
     * Retrieve rule categories
     *
     * @return \Affilae\Tracking\Model\ResourceModel\Category\Collection
     */
    public function getParentCategories()
    {
        if (is_null($this->_parentCategories)) {
            $this->_parentCategories = $this
                ->_categoryCollectionFactory
                ->create()
                ->addFieldToFilter(
                    'entity_id',
                    ['in' => $this->getCategories()]
                );
        }

        return $this->_parentCategories;
    }

    /**
     * Retrieve rule parent categories count
     *
     * @return int
     */
    public function getCategoriesCount()
    {
        return count($this->getParentCategories());
    }

    /**
     * Retrieve if is visible on store
     *
     * @return bool
     */
    public function isVisibleOnStore($storeId)
    {
        return $this->getIsActive() && array_intersect([0, $storeId], $this->getStoreIds());
    }

    /**
     * Retrieve rule creation date using format
     *
     * @param  string $format
     *
     * @return string
     */
    public function getCreationDate($format = 'Y-m-d H:i:s')
    {
        return date($format, strtotime($this->getData('creation_time')));
    }

    /**
     * Retrieve rule publish date using format
     *
     * @param  string $format
     *
     * @return string
     */
    public function getUpdateDate($format = 'Y-m-d H:i:s')
    {
        return date($format, strtotime($this->getData('update_time')));
    }
}
