<?php
/**
 * Copyright © 2016 Affilae. All rights reserved.
 */
namespace Affilae\Tracking\Observer;

use Magento\Framework\Event\ObserverInterface;

class SetConversionValueObserver implements ObserverInterface
{
    /**
     * @var \Affilae\Tracking\Helper\Data
     */
    protected $_helper;

    /**
     * @var \Magento\Sales\Model\ResourceModel\Order\Collection
     */
    protected $_collection;

    /**
     * Constructor
     *
     * @param \Affilae\Tracking\Helper\Data $helper
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Sales\Model\ResourceModel\Order\Collection $collection
     */
    public function __construct(
        \Affilae\Tracking\Helper\Data $helper,
        \Magento\Framework\Registry $registry,
        \Magento\Sales\Model\ResourceModel\Order\Collection $collection
    ) {
        $this->_helper     = $helper;
        $this->_collection = $collection;
        $this->_registry   = $registry;
    }

    /**
     * Set order informations into the registry
     *
     * @param \Magento\Framework\Event\Observer $observer
     *
     * @return \Magento\GoogleAdwords\Observer\SetConversionValueObserver
     */
    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        if (!($this->_helper->isTrackingEnabled())) {
            return $this;
        }

        $orderIds = array_filter($observer->getEvent()->getOrderIds());

        if (!$orderIds || !is_array($orderIds)) {
            return $this;
        }

        $this->_collection->addFieldToFilter(
            'entity_id',
            ['in' => $orderIds]
        );

        if ($this->_collection->count() > 0) {
            $conversionValue = 0;

            /** @var $order \Magento\Sales\Model\Order */
            foreach ($this->_collection as $order) {
                $conversionValue += $order->getBaseGrandTotal();
            }

            $this->registerOrderTotal($conversionValue);
            $this->registerOrderId($order);
            $this->registerOrderPayment($order);
            $this->registerOrderItems($order);
            $this->registerOrderCategories($order);
            $this->registerVouchers($order);
        }

        return $this;
    }

    /**
     * Register the total value of the order
     *
     * @param  double $conversionValue
     */
    public function registerOrderTotal($conversionValue)
    {
        $this->_registry->register(
            \Affilae\Tracking\Helper\Data::CONVERSION_VALUE_REGISTRY_NAME,
            $conversionValue
        );
    }

    /**
     * Register the increment id of the order
     *
     * @param \Magento\Sales\Model\Order $order
     */
    public function registerOrderId($order)
    {
        $this->_registry->register(
            \Affilae\Tracking\Helper\Data::CONVERSION_ID_REGISTRY_NAME,
            $order->getIncrementId()
        );
    }

    /**
     * Register the payment the order
     *
     * @param \Magento\Sales\Model\Order $order
     */
    public function registerOrderPayment($order)
    {
        $this->_registry->register(
            \Affilae\Tracking\Helper\Data::CONVERSION_PAYMENT_REGISTRY_NAME,
            $order
                ->getPayment()
                ->getMethodInstance()
                ->getTitle()
        );
    }

    /**
     * Register the items of the order
     *
     * @param \Magento\Sales\Model\Order $order
     */
    public function registerOrderItems($order)
    {
        $items = $order->getItems();

        $this->_registry->register(
            \Affilae\Tracking\Helper\Data::CONVERSION_ITEMS_REGISTRY_NAME,
            $items
        );
    }

    /**
     * Register the categories of the order
     *
     * @param \Magento\Sales\Model\Order $order
     */
    public function registerOrderCategories($order)
    {
        $items = $order->getItems();
        $categories = [];

        foreach ($items as $item) {
            $product = $item->getProduct();
            $productCategories = $product->getCategoryCollection();

            foreach ($productCategories as $productCategory) {
                $categoryId = $productCategory->getEntityId();

                if (!in_array($categoryId, $categories)) {
                    $categories[] = $categoryId;
                }

                break;
            }
        }

        $this->_registry->register(
            \Affilae\Tracking\Helper\Data::CONVERSION_CATEGORIES_REGISTRY_NAME,
            $categories
        );
    }

    /**
     * register the vouchers of the order
     *
     * @param \Magento\Sales\Model\Order $order
     */
    public function registerVouchers($order)
    {
        $vouchers = $order->getCouponCode();

        $this->_registry->register(
            \Affilae\Tracking\Helper\Data::CONVERSION_VOUCHERS,
            $vouchers
        );
    }
}
